/*
 * Copyright (C) 2022-2023 Jordan Bancino <@jordan:bancino.net>
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#ifndef TELODENDRIA_DB_H
#define TELODENDRIA_DB_H

#include <stddef.h>

#include <HashMap.h>
#include <Array.h>

typedef struct Db Db;
typedef struct DbRef DbRef;

extern Db *
 DbOpen(char *, size_t);

extern void
 DbClose(Db *);

extern DbRef *
 DbCreate(Db *, size_t,...);

extern int
 DbDelete(Db *, size_t,...);

extern DbRef *
 DbLock(Db *, size_t,...);

extern int
 DbUnlock(Db *, DbRef *);

extern int
 DbExists(Db *, size_t,...);

extern Array *
 DbList(Db *, size_t,...);

extern void
 DbListFree(Array *);

extern HashMap *
 DbJson(DbRef *);

#endif
